@@ -33,11 +33,11 @@
 #'
 #' @export
 #'
-#' @importFrom lifecycle deprecate_warn
+#' @importFrom lifecycle deprecate_stop
 initest_options <- function (keep_solutions = 5, psc_method = c("exact", "rr"), maxit = 10, maxit_pense_refinement = 5,
                              eps = 1e-6, psc_keep = 0.5, resid_keep_method = c("proportion", "threshold"),
                              resid_keep_prop = 0.6, resid_keep_thresh = 2, mscale_eps = 1e-8, mscale_maxit = 200) {
-  deprecate_warn('2.0.0', 'initest_options()', with = 'enpy_options()')
+  deprecate_stop('2.0.0', 'initest_options()', with = 'enpy_options()')
 
   resid_keep_method <- match.arg(resid_keep_method)
 
@@ -69,9 +69,9 @@ NULL
 #'
 #' @export
 #' @describeIn deprecated_en_options Superseded by [en_lars_options()].
-#' @importFrom lifecycle deprecate_warn
+#' @importFrom lifecycle deprecate_stop
 en_options_aug_lars <- function (use_gram = c("auto", "yes", "no"), eps = 1e-12) {
-  deprecate_warn('2.0.0', 'en_options_aug_lars()', with = 'en_lars_options()')
+  deprecate_stop('2.0.0', 'en_options_aug_lars()', with = 'en_lars_options()')
   en_lars_options()
 }
 
@@ -89,7 +89,7 @@ en_options_aug_lars <- function (use_gram = c("auto", "yes", "no"), eps = 1e-12)
 #' @describeIn deprecated_en_options Superseded by [en_dal_options()]
 en_options_dal <- function (maxit = 100, eps = 1e-8, eta_mult = 2, eta_start_numerator = 1e-2,
                             eta_start, preconditioner = c("approx", "none", "diagonal"), verbosity = 0) {
-  deprecate_warn('2.0.0', 'en_options_dal()', with = 'en_dal_options()')
+  deprecate_stop('2.0.0', 'en_options_dal()', with = 'en_dal_options()')
   en_dal_options(max_it = .as(maxit[[1L]], 'integer'), eta_multiplier = .as(eta_mult[[1L]], 'numeric'),
                  eta_start_conservative = .as(eta_start_numerator[[1L]], 'numeric'),
                  eta_start_aggressive = .as(eta_start_numerator[[1L]], 'numeric'))
@@ -124,10 +124,10 @@ en_options_dal <- function (maxit = 100, eps = 1e-8, eta_mult = 2, eta_start_num
 #'
 #' @export
 #'
-#' @importFrom lifecycle deprecate_warn
+#' @importFrom lifecycle deprecate_stop
 pense_options <- function (delta = 0.25, maxit = 1000, eps = 1e-6, mscale_eps = 1e-8, mscale_maxit = 200,
                            verbosity = 0, cc = NULL, en_correction = TRUE) {
-  deprecate_warn('2.0.0', 'pense_options()')
+  deprecate_stop('2.0.0', 'pense_options()')
   list(maxit = .as(maxit[[1L]], 'integer'), bdp = .as(delta[[1L]], 'numeric'), eps = .as(eps[[1L]], 'numeric'),
        mscale_opts = mscale_algorithm_options(max_it = maxit, eps = mscale_eps))
 }
@@ -158,10 +158,10 @@ pense_options <- function (delta = 0.25, maxit = 1000, eps = 1e-6, mscale_eps =
 #'
 #' @export
 #'
-#' @importFrom lifecycle deprecate_warn
+#' @importFrom lifecycle deprecate_stop
 mstep_options <- function (cc = 3.44, maxit = 1000, eps = 1e-6, adjust_bdp = FALSE, verbosity = 0,
                            en_correction = TRUE) {
-  deprecate_warn('2.0.0', 'mstep_options()')
+  deprecate_stop('2.0.0', 'mstep_options()')
   list(maxit = .as(maxit[[1L]], 'integer'))
 }
 
@@ -193,9 +193,9 @@ mstep_options <- function (cc = 3.44, maxit = 1000, eps = 1e-6, adjust_bdp = FAL
 #' @family deprecated functions
 #'
 #' @export
-#' @importFrom lifecycle deprecate_warn
+#' @importFrom lifecycle deprecate_stop
 enpy <- function(x, y, alpha, lambda, delta, cc, options, en_options) {
-  deprecate_warn('2.0.0', 'enpy()', with = 'enpy_initial_estimates()')
+  deprecate_stop('2.0.0', 'enpy()', with = 'enpy_initial_estimates()')
 
   new_enpy <- enpy_initial_estimates(x, y, alpha = alpha, lambda = lambda, bdp = delta, cc = cc)
   coefs <- unlist(lapply(new_enpy, function (est) {
